function [power, f] = GLOSA(allData, fmax, NBands, zoomIter, Na, plotIt, plotNum,lambda,zeta)
% Estimating shared frequencies in multi-time data using the GLOSA method
% Input variables:
%   allData - cell containing iddata object with all data sets used
%   fmin - lowest frequency of interest
%   fmax - highest frequency of interest
%   NBands - number of bands for the first iteration
%   zoomIter - number of zooming iterations
%   Na - number of peaks 
%   plotIt - boolean to decide wether to plot zooming steps or not
%   lambda, zeta - user defined weights for the cost function. 
% Output variables:
%   power - power of the Na highest peaks
%   f - frequencies of the Na highest peaks

if ~plotIt
%    plotData= [];
end
% Number of datasets
numSets = length(allData);

% How many bands should each active band be expanded to? 
expand = 2;
% Active threshold
minThresh = 1e-5;

% Width of each band
df = fmax/NBands;

% Starting frequency of each band
bandstart = (0:df:(fmax-df))';
% Ending frequency of each band
bandend = (df:df:fmax)';

bandcenter = mean([bandstart bandend],2);

%% Zooming step - solving with CVX
% Keep track of number of iterations
numItr = 0;
for i = 1:zoomIter
    numItr = numItr+1;
    % Perform one iteration of GLOSA. 
    [power,allSpect, ~] = oneIter(allData,bandstart, bandend,'best',lambda,zeta);
    % Which bands are active?
    active = find(power>minThresh);
    
    % Plot results.
    if plotIt
        figure(plotNum)
        plot(bandcenter, allSpect(:,1), '.', 'Color', [0 0.4470 0.7410]);
        hold on
        if numSets>1
            plot(bandcenter, allSpect(:,2:end), '.', 'HandleVisibility', 'off','Color', [0 0.4470 0.7410]);
        end
        plot(bandcenter, power, 'o', 'Color', [0.9290 0.6940 0.1250]);
        plot(mean([bandstart(active) bandend(active)],2), power(active,end), 'rx');
        xline(1/100);
        xline(1/41,'HandleVisibility', 'off');
        xline(1/23,'HandleVisibility', 'off');
        xline(1/19,'HandleVisibility', 'off');
        hold off
        legend('$\alpha$', '$\Phi$', '$\Phi_{active}$', 'Frequencies', 'interpreter', 'latex','location', 'northeast');
        title(['Active bands = ' num2str(length(active)) ', ' num2str(length(bandstart)*2*(2*numSets +1)) ' variables, iteration ' num2str(numItr)]);
        xlabel(['Bandwidth = ' num2str(mean(bandend-bandstart))]);
        g = gca;
        bands = zeros(2*length(bandstart),1);
        plotVal = zeros(2*length(bandstart),1);
        cntr = 1;
        for k = 1:length(bandstart)
            bands(cntr:cntr+1) = [bandstart(k) bandend(k)]';
            plotVal(cntr:cntr+1) = [g.YLim(2)-5 0]';
            cntr = cntr+2;
        end
        hold on
        stairs(bands, plotVal, 'HandleVisibility', 'off');
        hold off
    end
    % Change bands so that only the active bands are expanded. 
    if i < zoomIter
        bandstart = bandstart(active);
        [bandstart, bandend, bandcenter,df] = expandFF(bandstart, expand,df);
    end
end
%% Zooming done, now find bands with highest power and do gridless search
% Find bands with highest power
[~,sortIndx] = sort(power, 'descend');
% Number of bands left to find
K = Na;
maxFound = false;
while ~maxFound
    try
        cntr = 1;
        maxBands = zeros(K,1);
        % Highest band is certainly a maximum
        maxBands(1) = sortIndx(1);
        for i = 2:K
            dist = 0;
            % Require some separation between maxima (>1e-3) in this case.
            while min(dist)<1e-3
                cntr = cntr+1;
                candidateBand = sortIndx(cntr);
                dist = abs(bandcenter(candidateBand)-bandcenter(maxBands(1:i-1)));
            end
            % Highest power band which is still a unique max
            maxBands(i) = sortIndx(cntr);
        end
        % Na highest bands has been found.
        maxFound = true;
    catch
        K = K-1;
    end
end
% Indicate the maximums
if plotIt
    figure(plotNum)
    hold on
    plot(bandcenter(maxBands), power(maxBands), 'kx');
    hold off
    legend('$\alpha$', '$\Phi$', '$\Phi_{active}$', 'Frequencies', 'Maximum power', 'interpreter', 'latex','location', 'northeast');
end
% Perform grid-less search in the highest power bands
[f, power] = oneLastIter(allData, bandstart(maxBands)-df, bandend(maxBands)+df);
end
%%
function [bandstart, bandend, bandcenter,df] = expandFF(bandstart,expand,df)
df = df/expand;
numBands = length(bandstart);
tmpS = zeros(expand*numBands,1);
tmpE = zeros(expand*numBands,1);
cntr = 1;
for i = 1:numBands
    tmp =  bandstart(i):df:bandstart(i)+expand*df;
    tmpS(cntr:cntr+expand-1) = tmp(1:end-1)';
    tmpE(cntr:cntr+expand-1) = tmp(2:end)';
    cntr = cntr+expand;
end
bandstart = tmpS;
bandend = tmpE;
bandcenter = mean([bandstart bandend],2);
end

function [power,allPower,beta] = oneIter(allData,bandstart, bandend,precision,lambda,zeta)
% Number of sets
numSet = length(allData);
% Current number of frequency bands
K = length(bandstart);

% Pre-allocate spectral estimate
beta = zeros(2*K,numSet);
phi = zeros(2*K,1);
alpha = zeros(2*K, numSet);

% Initialize cvx, quiet true for no output
cvx_quiet(true)
cvx_begin
% Use higher precision in cvx when K is approaching Kmax
cvx_precision(precision);
% Test different solver
cvx_solver sedumi
variable beta(2*K,numSet) complex
variable alpha(2*K,numSet) nonnegative
variable phi(2*K,1) nonnegative
minimize(sumCostGLOSA(allData,bandstart,bandend,phi, beta,alpha,lambda,zeta,ones(2*K,1)))
subject to
alpha >= abs(beta);
cvx_end
power = (flip(phi(1:K)).^2+phi(K+1:end).^2)/2;
allPower = flip(alpha(1:K,:).^2+alpha(K+1:end,:).^2)/2;
end

function [f,power] = oneLastIter(allData,bandstart,bandend)
K = length(bandstart);
numSets = length(allData);

f0 = (bandend+bandstart)/2;
option = optimset('MaxIter', 4000*length(f0));
option.MaxFunEvals = 4000*length(f0);
option.TolX = 1e-13;
%option.Display = 'iter';
option.TolFun = 1e-13;
%option.PlotFcns = @optimplotfval;

f = fminsearch(@(param) frequencyFit(allData, param, bandstart, bandend), f0,option);

power = zeros(K,1);
for i = 1:numSets
    T = allData{i}.SamplingInstants;
    y = allData{i}.y;
    cntr = 1;
    dict = zeros(length(T), 2*K);
    for k = 1:K
        dict(:,cntr:cntr+1) = [cos(2*pi*f(k).*T) sin(2*pi*f(k).*T)];
        cntr = cntr+2;
    end
    dict = dict-mean(dict);
    amp = dict\y;
    power = power + amp(1:2:end).^2/2+amp(2:2:end).^2/2;
end
power = power./numSets;
end

function cost = frequencyFit(allData,freq, bandstart, bandend)
numSets = length(allData);
K = length(bandstart);
cost = 0;
for i = 1:numSets
    T = allData{i}.SamplingInstants;
    y = allData{i}.y;
    N = length(T);
    dict = zeros(N, 2*K);
    cntr = 1;
    for k = 1:K
        dict(:,cntr:cntr+1) = [cos(2*pi*freq(k).*T) sin(2*pi*freq(k).*T)];
        cntr = cntr+2;
    end
    dict = dict-mean(dict);
    est = dict*((dict'*dict)\(dict'*y));
    cost = cost + pow_pos(norm(y-est,2),2);
end
cost = cost + 1e9*any(freq(1:K)>bandend|freq(1:K)<bandstart);
end